@@ -20,18 +20,17 @@ export class ViewBasedJSONModel extends MutableDataModel {
   /**
    * Create a data model with static JSON data.
    *
-   * @param data - The dataset for initializing the data model.
+   * @param options - The options for creating the data model.
    */
-  constructor(data: DataSource) {
+  constructor(options: ViewBasedJSONModel.IOptions) {
     super();
-    this.updateDataset(data);
+    this.updateDataset(options);
 
     this._transformState = new TransformStateManager();
-    // Repaint grid on transform state update
-    // Note: This will also result in the `model-reset` signal being sent.
     this._transformState.changed.connect((sender, value) => {
-      this.currentView = this._transformState.createView(this._dataset);
-      this._transformSignal.emit(value);
+      // Repaint grid on transform state update
+      // Note: This will also result in the `model-reset` signal being sent.
+      this._transformStateChangedHandler(sender, value);
     });
     // first run: generate a list of indices corresponding
     // to the locations of multi-index arrays.
@@ -72,10 +71,10 @@ export class ViewBasedJSONModel extends MutableDataModel {
   /**
    * Sets the dataset for this model.
    *
-   * @param data - The data to be set on this data model
+   * @param options - The options for creating the model
    */
-  updateDataset(data: DataSource): void {
-    this._dataset = data;
+  updateDataset(options: ViewBasedJSONModel.IOptions): void {
+    this._dataset = options.datasource;
     this._updatePrimaryKeyMap();
     const view = new View(this._dataset);
     this.currentView = view;
@@ -85,7 +84,7 @@ export class ViewBasedJSONModel extends MutableDataModel {
    * Updates the primary key map, which provides a mapping from primary key
    * value to row in the full dataset.
    */
-  private _updatePrimaryKeyMap(): void {
+  protected _updatePrimaryKeyMap(): void {
     this._primaryKeyMap.clear();
 
     const primaryKey = this._dataset.schema.primaryKey as string[];
@@ -377,14 +376,14 @@ export class ViewBasedJSONModel extends MutableDataModel {
   /**
    * Get the current View for the model.
    */
-  protected get currentView(): View {
+  get currentView(): View {
     return this._currentView;
   }
 
   /**
    * Sets the provided View as the current View, then emits a changed signal.
    */
-  protected set currentView(view: View) {
+  set currentView(view: View) {
     this._currentView = view;
 
     this.emitChanged({ type: 'model-reset' });
@@ -422,12 +421,12 @@ export class ViewBasedJSONModel extends MutableDataModel {
   /**
    * Removes the provided transformation from the active state.
    *
-   * @param columnIndex - The index of the column state to be removed.
+   * @param column - The column state to be removed.
    *
    * @param transformType - The type of the transform to be removed from state.
    */
-  removeTransform(columnIndex: number, transformType: string): void {
-    this._transformState.remove(columnIndex, transformType);
+  removeTransform(column: string, transformType: string): void {
+    this._transformState.remove(column, transformType);
   }
 
   /**
@@ -450,12 +449,10 @@ export class ViewBasedJSONModel extends MutableDataModel {
   /**
    * Returns the transform metadata for the provided column.
    *
-   * @param columnIndex - The column index of the metadata to be retrieved.
+   * @param column - The column of the metadata to be retrieved.
    */
-  transformMetadata(
-    columnIndex: number,
-  ): TransformStateManager.IColumn | undefined {
-    return this._transformState.metadata(columnIndex);
+  transformMetadata(column: string): TransformStateManager.IColumn | undefined {
+    return this._transformState.metadata(column);
   }
 
   /**
@@ -463,11 +460,10 @@ export class ViewBasedJSONModel extends MutableDataModel {
    * the provided column index.
    *
    * @param region - The CellRegion to retrieve unique values for.
-   * @param columnIndex - The index to retrieve unique values for.
+   * @param column - The column to retrieve unique values for.
    */
-  uniqueValues(region: DataModel.CellRegion, columnIndex: number): any[] {
-    const columnName = this.metadata(region, 0, columnIndex)['name'];
-    return Array.from(new Set(this.dataset.data[columnName]));
+  uniqueValues(region: DataModel.CellRegion, column: string): Promise<any[]> {
+    return Promise.resolve(Array.from(new Set(this.dataset.data[column])));
   }
 
   /**
@@ -475,13 +471,13 @@ export class ViewBasedJSONModel extends MutableDataModel {
    * the provided column index after all transforms have been applied.
    *
    * @param region - The CellRegion to retrieve unique values for.
-   * @param columnIndex - The index to retrieve unique values for.
+   * @param column - The column to retrieve unique values for.
    */
   async uniqueValuesVisible(
     region: DataModel.CellRegion,
-    columnIndex: number,
+    column: string,
   ): Promise<any[]> {
-    return this._currentView.uniqueValues(region, columnIndex);
+    return this._currentView.uniqueValues(region, column);
   }
 
   get transformStateChanged(): ISignal<this, TransformStateManager.IEvent> {
@@ -495,8 +491,8 @@ export class ViewBasedJSONModel extends MutableDataModel {
     return this._transformState.activeTransforms;
   }
 
-  getFilterTransform(columnIndex: number): Transform.TransformSpec | undefined {
-    return this._transformState.getFilterTransform(columnIndex);
+  getFilterTransform(column: string): Transform.TransformSpec | undefined {
+    return this._transformState.getFilterTransform(column);
   }
 
   /**
@@ -597,8 +593,23 @@ export class ViewBasedJSONModel extends MutableDataModel {
     return this.currentView.getSchemaIndex(region, index);
   }
 
-  private _currentView: View;
-  private _transformSignal = new Signal<this, TransformStateManager.IEvent>(
+  /**
+   * Handler for transformState.changed events.
+   *
+   * @param sender - TransformStateManager
+   *
+   * @param value - Event.
+   */
+  protected _transformStateChangedHandler(
+    sender: TransformStateManager,
+    value: TransformStateManager.IEvent,
+  ) {
+    this.currentView = this._transformState.createView(this._dataset);
+    this._transformSignal.emit(value);
+  }
+
+  protected _currentView: View;
+  protected _transformSignal = new Signal<this, TransformStateManager.IEvent>(
     this,
   );
   private _dataSyncSignal = new Signal<this, ViewBasedJSONModel.IDataSyncEvent>(
@@ -618,6 +629,13 @@ export class ViewBasedJSONModel extends MutableDataModel {
  * The namespace for the `ViewBasedJSONModel` class statics.
  */
 export namespace ViewBasedJSONModel {
+  export interface IOptions {
+    /**
+     * The `DataSource` for the model.
+     */
+    datasource: DataSource;
+  }
+
   export interface IUpdateCellValuesOptions {
     /**
      * The `CellRegion` of the cell to be updated.
