@@ -124,6 +124,10 @@ export class FeatherGrid extends Widget {
 
     this._createGrid(options);
 
+    if (this.backboneModel) {
+      this.grid.copyConfig = this.backboneModel.get('copy_config');
+    }
+
     this._defaultRenderer = new TextRenderer({
       font: '12px sans-serif',
       textColor: Theme.getFontColor(),
@@ -879,14 +883,19 @@ export class FeatherGrid extends Widget {
       iconClass:
         'ipydatagrid-filterMenuIcon ipydatagrid-filterMenuIcon-sortAsc',
       execute: (args): void => {
-        const cellClick: FeatherGridContextMenu.CommandArgs =
+        const command: FeatherGridContextMenu.CommandArgs =
           args as FeatherGridContextMenu.CommandArgs;
+
         const colIndex = this._dataModel.getSchemaIndex(
-          cellClick.region,
-          cellClick.columnIndex,
+          command.region,
+          command.columnIndex,
         );
+
+        const column = this.dataModel.currentView.dataset.columns[colIndex];
+
         this._dataModel.addTransform({
           type: 'sort',
+          column,
           columnIndex: colIndex,
           desc: false,
         });
@@ -898,14 +907,19 @@ export class FeatherGrid extends Widget {
       iconClass:
         'ipydatagrid-filterMenuIcon ipydatagrid-filterMenuIcon-sortDesc',
       execute: (args) => {
-        const cellClick: FeatherGridContextMenu.CommandArgs =
+        const command: FeatherGridContextMenu.CommandArgs =
           args as FeatherGridContextMenu.CommandArgs;
+
         const colIndex = this._dataModel.getSchemaIndex(
-          cellClick.region,
-          cellClick.columnIndex,
+          command.region,
+          command.columnIndex,
         );
+
+        const column = this.dataModel.currentView.dataset.columns[colIndex];
+
         this._dataModel.addTransform({
           type: 'sort',
+          column,
           columnIndex: colIndex,
           desc: true,
         });
@@ -915,12 +929,16 @@ export class FeatherGrid extends Widget {
       label: 'Clear This Filter',
       mnemonic: -1,
       execute: (args) => {
-        const commandArgs = <FeatherGridContextMenu.CommandArgs>args;
-        const schemaIndex: number = this._dataModel.getSchemaIndex(
-          commandArgs.region,
-          commandArgs.columnIndex,
+        const command = <FeatherGridContextMenu.CommandArgs>args;
+
+        const colIndex = this._dataModel.getSchemaIndex(
+          command.region,
+          command.columnIndex,
         );
-        this._dataModel.removeTransform(schemaIndex, 'filter');
+
+        const column = this.dataModel.currentView.dataset.columns[colIndex];
+
+        this._dataModel.removeTransform(column, 'filter');
       },
     });
     commands.addCommand(
@@ -947,10 +965,19 @@ export class FeatherGrid extends Widget {
           'ipydatagrid-filterMenuIcon ipydatagrid-filterMenuIcon-filter',
         execute: (args) => {
           const commandArgs = <FeatherGridContextMenu.CommandArgs>args;
+
+          const colIndex = this._dataModel.getSchemaIndex(
+            commandArgs.region,
+            commandArgs.columnIndex,
+          );
+
+          const column = this.dataModel.currentView.dataset.columns[colIndex];
+
           this._filterDialog.open({
             x: commandArgs.clientX,
             y: commandArgs.clientY,
             region: commandArgs.region,
+            column,
             columnIndex: commandArgs.columnIndex,
             forceX: false,
             forceY: false,
@@ -968,10 +995,19 @@ export class FeatherGrid extends Widget {
           'ipydatagrid-filterMenuIcon ipydatagrid-filterMenuIcon-filter',
         execute: (args) => {
           const commandArgs = <FeatherGridContextMenu.CommandArgs>args;
+
+          const colIndex = this._dataModel.getSchemaIndex(
+            commandArgs.region,
+            commandArgs.columnIndex,
+          );
+
+          const column = this.dataModel.currentView.dataset.columns[colIndex];
+
           this._filterDialog.open({
             x: commandArgs.clientX,
             y: commandArgs.clientY,
             region: commandArgs.region,
+            column,
             columnIndex: commandArgs.columnIndex,
             forceX: false,
             forceY: false,
@@ -1020,11 +1056,15 @@ export class FeatherGrid extends Widget {
       mnemonic: 1,
       execute: (args) => {
         const commandArgs = <FeatherGridContextMenu.CommandArgs>args;
-        const schemaIndex: number = this._dataModel.getSchemaIndex(
+
+        const colIndex = this._dataModel.getSchemaIndex(
           commandArgs.region,
           commandArgs.columnIndex,
         );
-        this._dataModel.removeTransform(schemaIndex, 'sort');
+
+        const column = this.dataModel.currentView.dataset.columns[colIndex];
+
+        this._dataModel.removeTransform(column, 'sort');
       },
     });
     commands.addCommand(FeatherGridContextMenu.CommandID.ClearSelection, {
